;;; slovak-calendar.el -- Slovak calendar and holidays

;; Copyright (C) 1998 Milan Zamazal

;; Authors:    David auer <xsauer@hwlab.felk.cvut.cz>,
;;             Tibor imko <tibor.simko@fmph.uniba.sk>,
;;             Milan Zamazal <pdm@fi.muni.cz>
;;             Pavel Jank <Pavel@Janik.cz>
;;             Miroslav Vako <vasko@debian.cz>
;; Maintainer: Pavel Jank <Pavel@Janik.cz>
;; Version:    21.1
;; Keywords:   i18n, Slovak, calendar, holidays

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation, version 2 of the License.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You can find the GNU General Public License at
;; http://www.gnu.org/copyleft/gpl.html
;; or you can write to the Free Software Foundation, Inc., 675 Mass Ave.,
;; Cambridge, MA 02139, USA.

;;; Commentary:

;; This file defines and redefines various calendar things according to Slovak
;; national habits.

;;; History:

;; So long, so very long...

;;; Code:

;; Basic calendar settings according to Slovak habits
(setq european-calendar-style t
      calendar-week-start-day 1
      calendar-day-name-array
      ["Nedea" "Pondelok" "Utorok" "Streda" "tvrtok" "Piatok" "Sobota"]
      calendar-month-name-array
      ["Janur" "Februr" "Marec" "Aprl" "Mj" "Jn"
       "Jl" "August" "September" "Oktber" "November" "December"]
      display-time-24hr-format t
      cal-tex-24 t
      calendar-date-display-form '(day ". " month ". " year)
      calendar-time-display-form '(24-hours ":" minutes
					    (if time-zone " (")
					    time-zone
					    (if time-zone ")")))

;; This is really ugly hack :-((
(defun slovak-calendar-hack ()

  ;; Load it because of redefinition of some functions in it...
  (load "lunar")
  (defun lunar-phase-name (phase)
    "Name of lunar PHASE.
0 = new moon, 1 = first quarter, 2 = full moon, 3 = last quarter."
    (cond ((= 0 phase) "Nov")
	  ((= 1 phase) "Prv tvr")
	  ((= 2 phase) "Spln")
	  ((= 3 phase) "Posledn tvr")))

  ;; Load it because of redefinition of some functions in it...
  (load "cal-tex")
  ;; Do not allow short month names
  (defun calendar-month-name (month &optional width)
    "The name of MONTH.
Redefinition of the original function because of some difficulties with Slovak.
WIDTH is ignored because of this reason."
    (aref calendar-month-name-array (1- month)))

  ;; Redefine this function (add [slovak] to LaTeX header)
  (defun cal-tex-preamble (&optional args)
    "Insert the LaTeX preamble.
Preamble includes initial definitions for various LaTeX commands.
Optional ARGS are included."
    (set-buffer (get-buffer-create cal-tex-buffer))
    (erase-buffer)
    (insert "\\documentstyle")
    (if args
	(insert "[slovak," args "]")
      (insert "[slovak]"))
    (insert "{article}\n"
	    "\\hbadness 20000
\\hfuzz=1000pt
\\vbadness 20000
\\lineskip 0pt
\\marginparwidth 0pt
\\oddsidemargin  -2cm
\\evensidemargin -2cm
\\marginparsep   0pt
\\topmargin      0pt
\\textwidth      7.5in
\\textheight     9.5in
\\newlength{\\cellwidth}
\\newlength{\\cellheight}
\\newlength{\\boxwidth}
\\newlength{\\boxheight}
\\newlength{\\cellsize}
\\newcommand{\\myday}[1]{}
\\newcommand{\\caldate}[6]{}
\\newcommand{\\nocaldate}[6]{}
\\newcommand{\\calsmall}[6]{}
%
"))

  ;; Redefine this function (add new header comment to LaTeX file to use
  ;; csLaTeX command for TeXing).
  (defun cal-tex-end-document ()
    "Finish the LaTeX document.
Insert the trailer to LaTeX document, pop to LaTeX buffer, add 
informative header, and run HOOK."
    (cal-tex-e-document)
    (latex-mode)
    (pop-to-buffer cal-tex-buffer)
    ;; We should use CSLaTeX to process this buffer
    (setq tex-command "cslatex *")
    (goto-char (point-min))
    ;; Even if we save this buffer, we should still run CSLaTeX on it.
    (insert "%&cslatex\n")
    (cal-tex-comment "       Tento buffer je vsledkom cal-tex.el.")
    (cal-tex-comment "       Ak si chcete prezrie kalendr:")
    (cal-tex-comment "          M-x tex-buffer RET")
    (cal-tex-comment "          M-x tex-view  RET")
    (cal-tex-comment "       Ak si chcete vytlai kalendr:")
    (cal-tex-comment "          M-x tex-buffer RET")
    (cal-tex-comment "          M-x tex-print  RET")
    (goto-char (point-min))))

(add-hook 'initial-calendar-window-hook 'slovak-calendar-hack)

;; Slovak holidays
(setq general-holidays
      '((holiday-fixed
	 1 1 "De vzniku Slovenskej republiky")
	(holiday-fixed
	 1 1 "Nov rok")
	(holiday-fixed
	 1 6 "Zjavenie Pna (Traja Krli)")
	(holiday-fixed
	 5 1 "Sviatok prce")
	(holiday-fixed
	 5 8 "De vazstva nad faizmom (1945)")
	(holiday-fixed
	 7 5 "Sviatok svtho Cyrila a svtho Metoda")
	(holiday-fixed
	 8 29 "Vroie Slovenskho nrodnho povstania")
	(holiday-fixed
	 9 1 "De stavy Slovenskej republiky")
	(holiday-fixed
	 9 15 "Sedembolestn Panna Mria")
	(holiday-fixed
	 11 1 "Sviatok vetkch svtch")
	(holiday-fixed
	 12 24 "tedr de")
	(holiday-fixed
	 12 25 "Prv sviatok vianon")
	(holiday-fixed
	 12 26 "Druh sviatok vianon")))
(when emacs-cs-utils-calendar-names
  (load "slovak-calendar-names"))

;; Translate daylight savings time
(setq solar-holidays
      '((solar-equinoxes-solstices)
	(funcall
	 'holiday-sexp
	 calendar-daylight-savings-starts
	 '(format "Zana letn as"))
	(funcall
	 'holiday-sexp
	 calendar-daylight-savings-ends
	 '(format "Kon letn as"))))

;; Translate solar seasons
(setq solar-n-hemi-seasons
      '("Jarn rovnodennos" "Letn slnovrat" 
	"Jesenn rovnodennos" "Zimn slnovrat"))

;; Translate Christian holidays
(setq christian-holidays
      '((sk-holiday-easter-etc)
	(if all-christian-calendar-holidays
	    (holiday-greek-orthodox-easter))
	(if all-christian-calendar-holidays
	    (holiday-fixed 8 15 "Nanebevzatie Panny Mrie"))
	(if all-christian-calendar-holidays
	    (holiday-advent))
	(if all-christian-calendar-holidays
	    (holiday-julian 12 25 "Pravoslvne Vianoce"))))

;; Another replacing case.  Cut, paste & edit from `holidays.el' again.
;; Here we moreover slightly modify produced list, because in Slovak Republic
;; there is another set of mandatory Easter holidays.
(defun sk-holiday-easter-etc ()
  "List of dates related to Easter, as visible in calendar window."
  (if (and (> displayed-month 5) (not all-christian-calendar-holidays))
      nil ;; Ash Wednesday, Good Friday, and Easter are not visible.
    (let* ((century (1+ (/ displayed-year 100)))
	   (shifted-epact ;; Age of moon for April 5...
	    (% (+ 14 (* 11 (% displayed-year 19)) ;;     ...by Nicaean rule
		  (- ;; ...corrected for the Gregorian century rule
		   (/ (* 3 century) 4))
		  (/ ;; ...corrected for Metonic cycle inaccuracy.
		   (+ 5 (* 8 century)) 25)
		  (* 30 century)) ;;              Keeps value positive.
	       30))
	   (adjusted-epact ;;  Adjust for 29.5 day month.
	    (if (or (= shifted-epact 0)
		    (and (= shifted-epact 1) (< 10 (% displayed-year 19))))
		(1+ shifted-epact)
	      shifted-epact))
	   (paschal-moon ;; Day after the full moon on or after March 21.
	    (- (calendar-absolute-from-gregorian
		(list 4 19 displayed-year))
	       adjusted-epact))
	   (abs-easter (calendar-dayname-on-or-before
			0 (+ paschal-moon 7)))
	   (mandatory
	    (list
	     (list (calendar-gregorian-from-absolute (- abs-easter 2))
		   "Vekonon piatok")
	     (list (calendar-gregorian-from-absolute abs-easter)
		   "Vekonon nedea")
	     (list (calendar-gregorian-from-absolute (1+ abs-easter))
		   "Vekonon pondelok")))
	   (optional
	    (list
	     (list (calendar-gregorian-from-absolute (- abs-easter 49))
		   "Faiangov nedea")
	     (list (calendar-gregorian-from-absolute (- abs-easter 48))
		   "Faiangov pondelok")
	     (list (calendar-gregorian-from-absolute (- abs-easter 47))
		   "Faiangov utorok")
	     (list (calendar-gregorian-from-absolute (- abs-easter 46))
		   "Popolcov streda")
	     (list (calendar-gregorian-from-absolute (- abs-easter 42))
		   "Prv pstna nedea")
	     (list (calendar-gregorian-from-absolute (- abs-easter 14))
		   "Piata pstna nedea")
	     (list (calendar-gregorian-from-absolute (- abs-easter 7))
		   "Nedea utrpenia Pna (Kvetn)")
	     (list (calendar-gregorian-from-absolute (- abs-easter 3))
		   "Zelen tvrtok")
	     (list (calendar-gregorian-from-absolute (- abs-easter 2))
		   "Vek piatok")
	     (list (calendar-gregorian-from-absolute (- abs-easter 1))
		   "Svt (Vek) sobota")
	     (list (calendar-gregorian-from-absolute (+ abs-easter 35))
		   "Piata vekonon nedea")
	     (list (calendar-gregorian-from-absolute (+ abs-easter 39))
		   "Nanebovstpenie Pna")
	     (list (calendar-gregorian-from-absolute (+ abs-easter 49))
		   "Svtodun nedea (Turce)")
	     (list (calendar-gregorian-from-absolute (+ abs-easter 50))
		   "Svtodun pondelok")
	     (list (calendar-gregorian-from-absolute (+ abs-easter 56))
		   "Najsvtejia Trojica")
	     (list (calendar-gregorian-from-absolute (+ abs-easter 60))
		   "Najsvtejie Kristovo telo a krv (Boie Telo)")))
	   (output-list
	    (filter-visible-calendar-holidays mandatory)))
      (if all-christian-calendar-holidays
	  (setq output-list
		(append
		 (filter-visible-calendar-holidays optional)
		 output-list)))
      output-list)))

;;; *** Announce ***

(provide 'slovak-calendar)


;; Local variables:
;; coding: iso-latin-2
;; End:

;;; slovak-calendar.el ends here
